% PCDA double cross validation for variable selection with rank products

%I: X, classes, REP(number of repetitions of the double cross of rank
%   products)
%O: MIS: average number of misclassifications. Number of variables used is
%   given in nvars 

function [MIS,nvars]=RP2cv(X,classes,REP)

[objects,vars]=size(X);

for rep=1:REP
    %all inner, middle and outer loops follow same scheme
    outer1=randperm(20)';
    outer2=[20+randperm(19) 0]';
    m1=randperm(18);
    m2=randperm(18);

    % outer cross (20 fold)
    in=[1 2];
    for outer=1:10
        ind_outer=[outer1(in); outer2(in)];
        train =delsamps([0:39]', 1+[outer1(in); outer2(in)]);
        ind(outer,:)=[ind_outer' train'];
        ind_outer=ind_outer(ind_outer~=0);
        train=train(train~=0);
        test_outer=X(ind_outer,:);
        Tclass_outer=classes(ind_outer);
        X_outer=X(train,:);
        Xclass_outer=classes(train);

        %idx for next loop
        middle1=delsamps(outer1,in); %remove idx that has been used
        middle1=middle1(m1);%permute remaining idx
        middle2=delsamps(outer2,in);
        middle2=middle2(m2);
        % middle cross (19 fold)
        o=[1 2];
        for middle=1:9
            ind_middle=[middle1(o) middle2(o)];
            ind_middle=ind_middle(ind_middle~=0);
            test_middle=X(ind_middle,:);
            Tclass_middle=classes(ind_middle);
            ind_middle=[ind_outer; ind_middle];
            X_middle=delsamps(X,ind_middle);
            Xclass_middle=delsamps(classes,ind_middle);
            o=o+2;
            % build models using different numbers of components
            for pc=2:20
                [middleclass,D(:,middle,pc)]=PCDApred(test_middle,X_middle,Xclass_middle,pc);
                loss(middle,pc)=length(find(middleclass'~=[Tclass_middle]));
            end
        end % middle cross ends

        % find best number of components
        som=sum(loss);
        [m npc(outer)]=min(som(2:20));
        npc(outer)=npc(outer)+1;

        % rank products
        Dabs=abs(D);
        for i=1:9
            [y(i,:),j(i,:)]=sort(Dabs(:,i,npc(outer)));
        end
        for i=1:9
            for m=1:vars
                rank(i,m)=(vars+1)-find(j(i,:)==m);
            end
        end
        for m=1:vars
            rankpr(m)=prod(rank(:,m));
        end
        [a(:,outer,rep),b(:,outer,rep)]=sort(rankpr);

        %prediction erros for different # variables
        for n=[1 5 10]
            Xr=X(:,b(1:n,outer, rep));

            ind_outer=[outer1(in); outer2(in)];
            train =delsamps([0:39]', 1+[outer1(in); outer2(in)]);
            ind_outer=ind_outer(ind_outer~=0);
            train=train(train~=0);
            test_outer=Xr(ind_outer,:);
            Tclass_outer=classes(ind_outer);
            X_outer=Xr(train,:);
            Xclass_outer=classes(train);
            [class(n,ind_outer)]=LDApred(test_outer,X_outer,Xclass_outer);
        end

        for n=[15:5:50 60:10:150 170:20:vars]

            Xr=X(:,b(1:n,outer, rep));

            ind_outer=[outer1(in); outer2(in)];
            train =delsamps([0:39]', 1+[outer1(in); outer2(in)]);
            ind_outer=ind_outer(ind_outer~=0);
            train=train(train~=0);
            test_outer=Xr(ind_outer,:);
            Tclass_outer=classes(ind_outer);
            X_outer=Xr(train,:);
            Xclass_outer=classes(train);
            [class(n,ind_outer),d]=PCDApred(test_outer,X_outer,Xclass_outer,15);

        end
        in=in+2;

    end %outer cross ends
    for n=[1 5:5:50 60:10:150 170:20:vars]
        mis(rep,n)=length(find(class(n,:)'~=classes));
    end
end
nvars=[1 5:5:50 60:10:150 170:20:vars];
MIS=mean(mis(:,nvars),1);