function [sc,lo,s,maxpc]=scoloatno(data);
%
% Calculation of scores, loadings, singular values and 
% maximum possible PC's using the TNO-mode. Scoloatno.m
% is a subroutine of pcdatno.m.
%
% Input
% ------
% data    the (scaled) data matrix with the objects stored
%         in rows and the variables stored in columns.
%
% Output
% ------
% sc      PCA scores.
% lo      PCA loadings.
% s       singular values.
% maxpc   maximum number op PCs.
%
% I/O: [sc,lo,s,maxpc]=scoloatno(data)
%
% Written by Florian Wulfert.

[m,n] = size(data);
if n < m
  C		 = (data'*data)/(m-1);
  [u,s,v] = svd(C);
  temp2	 = (1:n)';
  escl	 = 1:n;
else
  C		 = (data*data')/(m-1);
  [u,s,v] = svd(C);
  v		 = data'*v;
  for i = 1:m
    v(:,i) = v(:,i)/norm(v(:,i));
  end
  temp2	 = (1:m)';
  escl 	 = 1:m;
end

lo		=	v*sqrt(s*(m-1));
sc		=	data*v/sqrt(s*(m-1));
maxpc	=	min([m,n,max(find(diag(s)>100*eps))]);